# =====================================
# generator=datazen
# version=3.1.4
# hash=7bf0794b4628951026a3aca53df62fb3
# =====================================
"""
instagram-bucketizer - Package's default entry-point.
"""

# built-in
import sys

# internal
from instagram_bucketizer.entry import main

if __name__ == "__main__":
    sys.exit(main(sys.argv))
