# =====================================
# generator=datazen
# version=3.1.4
# hash=647f4d4150952038b0f7695c6f3bbca4
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = (
    "Small project to assist in research, categorizing instagram emoji usage"
)
PKG_NAME = "instagram-bucketizer"
VERSION = "0.1.0"
